/* remove a spell from a character's innate linked list */
void innate_remove(struct char_data * ch, struct innate_node * inn)
{
  struct innate_node *temp;

  if (ch->innate == NULL) {
    core_dump();
    return;
  }

  REMOVE_FROM_LIST(inn, ch->innate, next);
  free(inn);
}

void innate_add(struct char_data * ch, int innate, int timer)
{
  struct innate_node * inn;

  CREATE(inn, struct innate_node, 1);
  inn->timer = timer;
  inn->spellnum = innate;
  inn->next = ch->innate;
  ch->innate = inn;
}

/* Returns true if the spell/skillnum is innate to the character
   (as opposed to just practiced).
 */
int is_innate(struct char_data *ch, int spellnum) 
{
  switch(spellnum) {
  case SPELL_FAERIE_FIRE:
    if(GET_RACE(ch) == RACE_DROW)
      return TRUE;
    break;
  }
  return FALSE;
}

/* Adds a node to the linked list with a timer which indicates
   that the innate skill/spell is NOT ready to be used.

   is_innate_ready(...) should be called first to determine if
   there is already a timer set for the spellnum.
*/
void add_innate_timer(struct char_data *ch, int spellnum) 
{
  int timer = 6; /* number of ticks */

  switch(spellnum) {
    case SKILL_DOUBLE_BACKSTAB:
      timer = 3;
      break;
    case SPELL_FAERIE_FIRE:
      timer = 6;   
      break;
    case SKILL_KATA:
      timer = 6;
      break;
    case SKILL_LAY_HANDS:
      timer = 12;
      break;
  }
  if(is_innate_ready(ch, spellnum)) {
    innate_add(ch, spellnum, timer);
  } else {
    send_to_char(ch, "BUG!\r\n");
  }
}

/* returns FALSE if the spell is found in the innate linked list. 
  This just means the ability is ready IF they can even get the
  ability. (see is_innate). 
*/
int is_innate_ready(struct char_data *ch, int spellnum)
{
  struct innate_node *inn, *next_inn;

  for(inn = ch->innate; inn; inn = next_inn) {
    next_inn = inn->next;
    if(inn->spellnum == spellnum)
      return FALSE;
  }
  return TRUE;
}

/* called when a player enters the game to set permanent affects.
   Usually these will stay set, but certain circumstances will
   cause them to wear off (ie. removing eq with a perm affect).
*/
void add_innate_affects(struct char_data *ch) 
{
  switch(GET_RACE(ch)) {
  case RACE_ELF:
  case RACE_DROW:
  case RACE_DWARF:
  case RACE_HALFELF:
  case RACE_ILLITHID:
  case RACE_HALFLING:
    affect_modify(ch, APPLY_NONE, 0, AFF_INFRAVISION, TRUE);
    break;
  }
  affect_total(ch);
}

/* Called to update the innate timers */
void update_innate(struct char_data *ch)
{
  struct innate_node *inn, *next_inn;

  for (inn = ch->innate; inn; inn = next_inn) {
    next_inn = inn->next;
    if (inn->timer > 0) { 
      inn->timer--;
    } else {
      switch(inn->spellnum) {
      case SKILL_KATA:
        send_to_char(ch, "You feel more in tune with your special ability.\r\n");
        break;
      case SKILL_LAY_HANDS:
        send_to_char(ch, "Your special healing abilities have returned.\r\n");
        break;
      default:
        send_to_char(ch, "You are now able to use your innate %s again.\r\n", spell_info[inn->spellnum].name);
        break;
      }  
      innate_remove(ch, inn);
    }
  } 
}
