/* You will need to change mana_max to spell_level in the spello struct/declaration 
void spello(int spl, const char *name, int spell_level, int damage_type,
       int mana_change, int minpos, int targets, int violent, int routines, const char *wearoff)


example:
                                           
  spello(SPELL_CHILL_TOUCH, "chill touch", 2, DAM_COLD, 3, POS_FIGHTING,
    TAR_CHAR_ROOM | TAR_FIGHT_VICT, TRUE, MAG_DAMAGE | MAG_AFFECTS,
    "You feel your strength return.");

The 3rd argument is the spell level.

*/


/*
 * do_cast is the entry point for PC-casted spells.  It parses the arguments,
 * determines the spell number and finds a target, throws the die to see if
 * the spell can be cast, checks for sufficient mana and subtracts it, and
 * passes control to cast_spell().
 */

ACMD(do_cast)
{
  struct char_data *tch = NULL;
  struct obj_data *tobj = NULL;
  char *s, *t, buffer[10], export[256];
  int spellnum, i, target = 0, innate = FALSE;

  if (IS_NPC(ch))
    return;

  /* get: blank, spell name, target name */
  s = strtok(argument, "'");

  if (s == NULL) {
    send_to_char(ch, "Cast what where?\r\n");
    return;
  }
  s = strtok(NULL, "'");
  if (s == NULL) {
    send_to_char(ch, "Spell names must be enclosed in the Holy Magic Symbols: '\r\n");
    return;
  }
  t = strtok(NULL, "\0");

  /* capture export for multiple arguments before t gets nuked later on */
  sprintf(export, "%s", t);

  /* spellnum = search_block(s, spells, 0); */
  spellnum = find_skill_num(s);

  sprintf(buffer, "%d", spellnum);

  if ((spellnum < 1) || (spellnum > MAX_SPELLS)) {
    send_to_char(ch, "Cast what?!?\r\n");
    return;
  }
  if(!is_innate(ch, spellnum)) {
    if ((!IS_NPC(ch) && GET_SKILL(ch, spellnum) <= 0) 
	  || (IS_NPC(ch) && GET_NPC_SKILL(ch, spellnum) <= 0)) {
      send_to_char(ch, "You are unfamiliar with that spell.\r\n");
      return;
    }
  }
  /* Find the target */
  if (t != NULL) {
    one_argument(strcpy(arg, t), t);
    skip_spaces(&t);
  }
  if (IS_SET(SINFO.targets, TAR_IGNORE)) {
    target = TRUE;
  } else if (t != NULL && *t) {
    if (!target && (IS_SET(SINFO.targets, TAR_CHAR_ROOM))) {
      if ((tch = get_char_vis(ch, t, NULL, FIND_CHAR_ROOM)) != NULL)
	target = TRUE;
    }
    if (!target && IS_SET(SINFO.targets, TAR_CHAR_WORLD))
      if ((tch = get_char_vis(ch, t, NULL, FIND_CHAR_WORLD)) != NULL)
	target = TRUE;

    if (!target && IS_SET(SINFO.targets, TAR_OBJ_INV))
      if ((tobj = get_obj_in_list_vis(ch, t, NULL, ch->carrying)) != NULL)
	target = TRUE;

    if (!target && IS_SET(SINFO.targets, TAR_OBJ_EQUIP)) {
      for (i = 0; !target && i < NUM_WEARS; i++)
	if (GET_EQ(ch, i) && isname(t, GET_EQ(ch, i)->name)) {
	  tobj = GET_EQ(ch, i);
	  target = TRUE;
	}
    }
    if (!target && IS_SET(SINFO.targets, TAR_OBJ_ROOM))
      if ((tobj = get_obj_in_list_vis(ch, t, NULL, world[IN_ROOM(ch)].contents)) != NULL)
	target = TRUE;

    if (!target && IS_SET(SINFO.targets, TAR_OBJ_WORLD))
      if ((tobj = get_obj_vis(ch, t, NULL)) != NULL)
	target = TRUE;

	/* Export allows arguments from functions */
    if ( !target && IS_SET( SINFO.targets, TAR_EXPORT ) )
    {
      strcpy( export_variable, export );
      target = TRUE;
    }

  } else {			/* if target string is empty */
    if (!target && IS_SET(SINFO.targets, TAR_FIGHT_SELF))
      if (FIGHTING(ch) != NULL) {
	tch = ch;
	target = TRUE;
      }
    if (!target && IS_SET(SINFO.targets, TAR_FIGHT_VICT))
      if (FIGHTING(ch) != NULL) {
	tch = FIGHTING(ch);
	target = TRUE;
      }
    /* if no target specified, and the spell isn't violent, default to self */
    if (!target && IS_SET(SINFO.targets, TAR_CHAR_ROOM) &&
	!SINFO.violent) {
      tch = ch;
      target = TRUE;
    }
    if (!target) {
      if (IS_SET( SINFO.targets, TAR_EXPORT ))
        sprintf(buf, "This spell requires arguments... \r\n");
      else {
        sprintf(buf, "Upon %s should the spell be cast?\r\n",
	 IS_SET(SINFO.targets, TAR_OBJ_ROOM | TAR_OBJ_INV | TAR_OBJ_WORLD | TAR_OBJ_EQUIP) ? "what" : "who");
      }
      send_to_charf(buf, ch);
      return;
    }
  }

  if (target && (tch == ch) && SINFO.violent) {
    send_to_char(ch, "You shouldn't cast that on yourself -- could be bad for your health!\r\n");
    return;
  }
  if (!target) {
    send_to_char(ch, "Cannot find the target of your spell!\r\n");
    return;
  }

  if(is_innate(ch, spellnum))
    innate = TRUE;

  /* if spell NOT memorized */
  if ((GET_LEVEL(ch) < LVL_IMMORT) && !IS_NPC(ch) && (!is_memorized(ch, spellnum, TRUE))) {
    /* innate, not memorized, not ready */
    if(innate && (!is_innate_ready(ch, spellnum))) {
      send_to_char(ch, "You must wait a while before using that innate spell.\r\n");
      return;
    /* not innate, not memorized */
    } else if(!innate) {
      send_to_char(ch, "You do not have that spell memorized!\r\n");
      return;
    }
    /* otherwise they can cast innately */
  }

  if(!innate)
    improve_skill(ch, spellnum);

  /* You throws the dice and you takes your chances.. 101% is total failure */
  
  if (!innate && ((!IS_NPC(ch) && (number(0, 101) > GET_SKILL(ch, spellnum))) 
	  || (IS_NPC(ch) && (number(0, 101) > GET_NPC_SKILL(ch, spellnum))))) {
	WAIT_STATE(ch, PULSE_VIOLENCE);
  
    if (!tch || !skill_message(0, ch, tch, spellnum))
      send_to_charf("You lost your concentration!\r\n", ch);
    
    /*remove spell from memory here, remem if automem is on*/
    if ((GET_LEVEL(ch) >= LVL_IMMORT) || IS_NPC(ch)) return;

    if(!innate) {   
      do_mem_spell(ch, buffer, SCMD_FORGET, 0);

      if (PRF_FLAGGED(ch, PRF_AUTOMEM)) 
        do_mem_spell(ch, buffer, SCMD_MEMORIZE, 0);
    } else {
      add_innate_timer(ch, spellnum);
    }
	
    if (SINFO.violent && tch && IS_NPC(tch))
        hit(tch, ch, TYPE_UNDEFINED);
  } else { /* cast spell returns 1 on success; unmemorize/autoremem & set waitstate */
    if (cast_spell(ch, tch, tobj, spellnum)) {
      WAIT_STATE(ch, PULSE_VIOLENCE);

      /*take away the mem spell from memory, remem if automem is on*/
      if ((GET_LEVEL(ch) >= LVL_IMMORT) || IS_NPC(ch)) return;
        
      if(!innate) {
        do_mem_spell(ch, buffer, SCMD_FORGET, 0);
	  
        if (PRF_FLAGGED(ch, PRF_AUTOMEM)) 
          do_mem_spell(ch, buffer, SCMD_MEMORIZE, 0);
      } else {
        add_innate_timer(ch, spellnum);
      }
    }
  }
}

